@@ -1,10 +1,11 @@
 from naomi.eeprom import NaomiEEPRom, NaomiEEPRomException
 from naomi.generic_patch import force_freeplay, force_no_attract_sound
-from naomi.rom import NaomiRom, NaomiExecutable, NaomiRomSection, NaomiRomException
+from naomi.rom import NaomiRom, NaomiRomRegionEnum, NaomiExecutable, NaomiRomSection, NaomiRomException
 from naomi.settings_patch import NaomiSettingsPatcher, NaomiSettingsPatcherException, get_default_trojan
 
 __all__ = [
     "NaomiRom",
+    "NaomiRomRegionEnum",
     "NaomiExecutable",
     "NaomiRomSection",
     "NaomiRomException",
